//
//  CPDebug.h
//  CPCore
//
//  Created by Brad Miller on 11/20/05.
//  Copyright 2005 CynicalPeak. All rights reserved.
//  Logging inpired and partially borred from http://www.borkware.com/rants/agentm/mlog/

#import <Cocoa/Cocoa.h>


// Replacement for NSLog, use CPLog in the same manner as NSLog
// Adds a file and line number stamp to the log message.  Also allows for logging to be turned on by a user if we need the feedback.
//
// Messaging is off by default.  Can be turned on by calling setStandardLog:YES
// Can also be turned on by a user default key, CPStandardLog, for the app set to the debug level.  Easy access for users to turn it on if we need log information for debugging. 
// Inteded use is to put the following at the start of the app:
//
// #ifdef CPLOG
//		[CPDebug setStandardLog:YES];
// #endif
//
// and set -DCPLOG=1 in the target's other c flags section


// Debug mode adds a debug menu to the app that has a console and the ability to save the log to a file. 
// Can be turned on by setting calling setDebug:YES
// Can also be turned on by a user default key, CPDebug, for the app set to the debug level.  Easy access for users to turn it on if we need log information for debugging. If both standard log and debug are turned on, the level set in debug takes precedence. 

enum 
{
	CPDebugError = 1UL,
	CPDebugDebug = 1UL << 1,
    CPDebugWarning = 1UL << 2,
	CPDebugAlert = 1UL << 3,
	CPDebugCritical = 1UL << 4,
	CPDebugEmergency = 1UL << 5,
	CPDebugTrace = 1UL << 7,
	CPDebugTimedTrace = 1UL << 8,
	CPDebugAllTrace = 3UL << 7,
	CPDebugInfo = 1UL << 9,
	CPDebugNotice = 1UL << 10,
	CPDebugInfoNotice = 3UL << 9,
	CPDebugAll = NSUIntegerMax
};
typedef NSUInteger CPDebugLevel;

#define MAXTIMERS 5

@interface CPDebug : NSObject 
{
	CPDebugLevel debugLevel;
	BOOL standardLog;
	BOOL debug;
	
	NSTimeInterval startTime[MAXTIMERS];
}

+ (CPDebug *)standardDebug;

- (void)logFile:(char*)sourceFile lineNumber:(NSInteger)lineNumber logLevel:(CPDebugLevel)level format:(NSString*)format, ...;
- (void)logMessage:(NSString *)message logLevel:(CPDebugLevel)level;

- (BOOL)standardLog;
- (void)setStandardLog:(BOOL)logOn;

- (BOOL)debug;
- (void)setDebug:(BOOL)debugOn;

- (CPDebugLevel)debugLevel;
- (void)setDebugLevel:(CPDebugLevel)newLevel;

- (void)startTimer:(NSInteger)timerNumber;
- (void)endTimer:(NSInteger)timerNumber file:(char*)sourceFile lineNumber:(NSInteger)lineNumber message:(NSString*)message;

// Adds an item to the debug menu.  The item should be an NSMenuItem that has its target and selector set.
+ (void)addDebugMenuItem:(NSMenuItem *)item;

// Adds a group of items to the debug menu.  All items should be NSMenuItems that have their taget and selector set.
// seperator menu items will surround the group
+ (void)addDebugMenuItems:(NSArray *)items;

// Removes all added items from the debug menu.  Console and save log will remain.
+ (void)clearDebugMenu;

// Loads the feedback menu item.  The menu item is automatically loaded if the debug menu is shown.
+ (void)loadFeedbackMenu;

@end
